#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""List pip pacakges in a given conda environments."""

# yapf: disable

# Standard library imports
import json
import os.path as osp


# yapf: enable

PIP = True
try:
    from pip._internal.utils.misc import get_installed_distributions
except ImportError:
    PIP = False
except Exception:
    PIP = False

PIP_LIST_SCRIPT = osp.realpath(__file__).replace('.pyc', '.py')


def main():
    """Use pip to find pip installed packages in a given prefix."""
    pip_packages = {}
    if PIP:
        for package in get_installed_distributions():
            name = package.project_name
            version = package.version
            full_name = "{0}-{1}-pip".format(name.lower(), version)
            pip_packages[full_name] = {'version': version}
    data = json.dumps(pip_packages)
    print(data)


if __name__ == '__main__':  # pragma: no cover
    try:
        main()
    except Exception as inst:
        # Something went wrong, so the package list is the empty list
        print('{}')



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Projects Tab."""

# yapf: disable

from __future__ import absolute_import, division, print_function

# Third party imports
from qtpy.QtCore import Qt, Signal
from qtpy.QtWidgets import QHBoxLayout, QMenu, QVBoxLayout

# Local imports
from anaconda_navigator.api.anaconda_api import AnacondaAPI
from anaconda_navigator.utils import constants as C
# from anaconda_navigator.utils.logs import logger
from anaconda_navigator.utils.styles import load_style_sheet
from anaconda_navigator.widgets import (ButtonToolNormal,
                                        FrameEnvironmentsList,
                                        FrameEnvironmentsPackages,
                                        FrameTabHeader, WidgetBase)
from anaconda_navigator.widgets.helperwidgets import (ButtonToggleCollapse,
                                                      LineEditSearch)
from anaconda_navigator.widgets.lists.projects import (ListItemEnv,
                                                       ListWidgetEnv)
from anaconda_navigator.widgets.projects import ProjectsWidget


# yapf: enable

# TODO: Have a single ListWidgetBase that incorporates needed and common
# functionality between projects and envs


class ProjectsTab(WidgetBase):
    """Projects management tab."""
    # name, path, sender
    sig_item_selected = Signal(object, object, object)

    sig_create_requested = Signal()
    sig_import_requested = Signal()
    sig_remove_requested = Signal()
    sig_upload_requested = Signal()

    sig_login_requested = Signal()
    sig_ready = Signal()

    #    sig_apps_changed = Signal(str)
    #    sig_apps_updated = Signal()
    #    sig_project_updated = Signal()
    #    sig_status_updated = Signal(str, int, int, int)

    def __init__(self, parent=None):
        super(ProjectsTab, self).__init__(parent)

        # Variables
        self.api = AnacondaAPI()
        self.current_project = None
        self.style_sheet = None
        self.projects = None

        # Widgets
        self.frame_list = FrameEnvironmentsList(self)
        self.frame_widget = FrameEnvironmentsPackages(self)
        self.frame_header_left = FrameTabHeader()
        self.frame_header_right = FrameTabHeader()
        self.button_create = ButtonToolNormal(text="Create")
        self.button_import = ButtonToolNormal(text="Import")
        self.button_remove = ButtonToolNormal(text="Remove")
        self.button_toggle_collapse = ButtonToggleCollapse()
        self.list = ListWidgetEnv()
        self.widget = ProjectsWidget()
        self.menu_list = QMenu()
        self.text_search = LineEditSearch()

        # Widgets setup
        self.frame_list.is_expanded = True
        self.list.setContextMenuPolicy(Qt.CustomContextMenu)
        self.text_search.setPlaceholderText("Search Projects")
        self.button_create.setObjectName("create")
        self.button_import.setObjectName("import")
        self.button_remove.setObjectName("remove")

        # Layouts
        layout_header_left = QVBoxLayout()
        layout_header_left.addWidget(self.text_search)
        self.frame_header_left.setLayout(layout_header_left)

        layout_buttons = QHBoxLayout()
        layout_buttons.addWidget(self.button_create)
        layout_buttons.addWidget(self.button_import)
        layout_buttons.addWidget(self.button_remove)

        layout_list_buttons = QVBoxLayout()
        layout_list_buttons.addWidget(self.frame_header_left)
        layout_list_buttons.addWidget(self.list)
        layout_list_buttons.addLayout(layout_buttons)
        self.frame_list.setLayout(layout_list_buttons)

        layout_widget = QHBoxLayout()
        layout_widget.addWidget(self.widget)
        self.frame_widget.setLayout(layout_widget)

        layout_main = QHBoxLayout()
        layout_main.addWidget(self.frame_list, 10)
        layout_main.addWidget(self.button_toggle_collapse, 1)
        layout_main.addWidget(self.frame_widget, 30)
        self.setLayout(layout_main)

        # Signals
        self.button_toggle_collapse.clicked.connect(self.expand_collapse)
        self.button_create.clicked.connect(self.sig_create_requested)
        self.button_import.clicked.connect(self.sig_import_requested)
        self.button_remove.clicked.connect(self.sig_remove_requested)

        self.list.sig_item_selected.connect(self._item_selected)
        self.text_search.textChanged.connect(self.filter_list)
        self.widget.sig_login_requested.connect(self.sig_login_requested)

        self.refresh()

    # --- Setup methods
    # -------------------------------------------------------------------------

    def setup(self, projects):
        """Setup tab content and populates the list of projects."""
        self.set_projects(projects=projects)

    def set_projects(self, projects, current_project=None):
        """Populate the list of projects."""
        self.projects = projects
        if current_project is None:
            for (proj_path, proj_name) in projects.items():
                current_project = proj_path
                break

        self.list.clear()
        self.current_project = current_project
        selected_item_row = 0
        for i, (proj_path, proj_name) in enumerate(projects.items()):
            item = ListItemEnv(prefix=proj_path, name=proj_name)

            if proj_path == self.current_project:
                selected_item_row = i
            self.list.addItem(item)

        loading = False
        self.list.setCurrentRow(selected_item_row, loading=loading)
        self.set_project_widget(self.current_project)
        self.filter_list()

    def set_project_widget(self, project_path):
        """Set the project widget."""
        if project_path is None:
            # Disabled widget
            pass
        else:
            self.widget.load_project(project_path)
        self.refresh()
        self.sig_ready.emit()

    def before_delete(self):
        """Prerpare widget before delete."""
        self.widget.before_delete()

    def update_brand(self, brand):
        """Update service brand."""
        self.widget.update_brand(brand)

    # --- Common Helpers (# FIXME: factor out to common base widget)
    # -------------------------------------------------------------------------
    def _item_selected(self, item):
        """Callback to emit signal as user selects an item from the list."""
        prefix = item.prefix()
        self.current_project = prefix
        self.set_loading(prefix)
        self.sig_item_selected.emit(item.name, prefix, C.TAB_PROJECTS)

    def add_temporal_item(self, name):
        """Creates a temporal item on list while creation becomes effective."""
        item_names = [item.name for item in self.list.items()]
        item_names.append(name)
        index = list(sorted(item_names)).index(name) + 1
        item = ListItemEnv(name=name)
        self.list.insertItem(index, item)
        self.list.setCurrentRow(index)
        self.list.scrollToItem(item)
        item.set_loading(True)

    def expand_collapse(self):
        """Expand or collapse the list selector."""
        if self.frame_list.is_expanded:
            self.frame_list.hide()
            self.frame_list.is_expanded = False
        else:
            self.frame_list.show()
            self.frame_list.is_expanded = True

    def filter_list(self, text=None):
        """Filter items in list by name."""
        text = self.text_search.text().lower()
        for i in range(self.list.count()):
            item = self.list.item(i)
            item.setHidden(text not in item.name.lower())

            if not item.widget.isVisible():
                item.widget.repaint()

    def ordered_widgets(self, next_widget=None):
        """Return a list of the ordered widgets."""
        ordered_widgets = [self.text_search]
        ordered_widgets += self.list.ordered_widgets()
        ordered_widgets += [self.button_create, self.button_import, self.button_remove]
        ordered_widgets += self.widget.ordered_widgets()
        return ordered_widgets

    def refresh(self):
        """Refresh the enabled/disabled status of the widget and subwidgets."""
        projects = self.projects
        active = bool(projects)
        if not active:
            self.widget.clear()
        self.widget.setVisible(active)
        self.button_remove.setEnabled(active)
        self.widget.setEnabled(active)

    def set_loading(self, prefix=None, value=True):
        """Set the item given by `prefix` to loading state."""
        for row, item in enumerate(self.list.items()):
            if item.prefix == prefix:
                item.set_loading(value)
                self.list.setCurrentRow(row)
                break

    def set_widgets_enabled(self, value):
        """Change the enabled status of widgets and subwidgets."""
        self.list.setEnabled(value)
        self.button_create.setEnabled(value)
        self.button_import.setEnabled(value)
        self.button_remove.setEnabled(value)
        self.widget_projects.set_widgets_enabled(value)
        if value:
            self.refresh()

    @staticmethod
    def update_status(action=None, message=None, value=0, max_value=0):
        """Update status bar."""
        # TODO:!

    def update_style_sheet(self, style_sheet=None):
        """Update custom CSS style sheet."""
        if style_sheet is None:
            self.style_sheet = load_style_sheet()
        else:
            self.style_sheet = style_sheet

        self.setStyleSheet(self.style_sheet)


def local_test():  # pragma: no cover
    """Run local test for project tab."""
    from anaconda_navigator.utils.qthelpers import qapplication
    app = qapplication()
    w = ProjectsTab()
    w.update_style_sheet()
    w.showMaximized()
    app.exec_()


if __name__ == "__main__":  # pragma: no cover
    local_test()



#!/opt/concourse/worker/volumes/live/ce16c7e8-1b9d-4ed5-71a0-f56450a96636/volume/anaconda-navigator_1619554649950/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/opt/concourse/worker/volumes/live/ce16c7e8-1b9d-4ed5-71a0-f56450a96636/volume/anaconda-navigator_1619554649950/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/bin/anaconda-navigator"])



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Tests for environments tab."""

# yapf: disable

# Standard library imports
import sys

# Third party imports
from qtpy.QtCore import Qt
import pytest

# Local imports
from anaconda_navigator.api.conda_api import CondaAPI
from anaconda_navigator.utils.fixtures import tmpfile, tmpfolder
from anaconda_navigator.widgets.dialogs import MessageBoxError
from anaconda_navigator.widgets.tabs.environments import EnvironmentsTab


# yapf: enable

tmpfile
tmpfolder
PY3 = sys.version_info >= (3, 4)
xfail = pytest.mark.xfail


@pytest.fixture()
def env_tab(qtbot, tmpfile):
    widget = EnvironmentsTab()
    qtbot.addWidget(widget)
    widget.show()
    widget.setup_tab(metadata={})
    widget.load_environment()

    with qtbot.waitSignal(widget.sig_status_updated) as blocker:
        blocker

    return widget, qtbot, tmpfile


MessageBoxError.exec_ = lambda *args: True


class TestEnvironmentsTab:
    def package_version(self, pkg, name='root'):
        api = CondaAPI()
        return api.package_version(name=name, pkg=pkg, build=True)

    def remove_env(self, widget):
        worker = widget.packages_widget.remove_environment(name='navigatortest')
        worker.communicate()  # run create

    @xfail
    def test_bad_create(self, env_tab):  # analysis:ignore
        widget, qtbot, tmpfile = env_tab

        with open(tmpfile, 'w') as f:
            raw = "name: navigatortest\ndependencies:\n- not-real=0.0.0=py36_0"
            f.write(raw)
        worker = widget.packages_widget.import_yaml(name="navigatortest", yaml=tmpfile)

        with qtbot.waitSignal(widget.sig_error_popped_up, timeout=5000):
            with qtbot.waitSignal(worker.sig_finished, timeout=5000):
                worker.name = "navigatortest"
                worker.sig_finished.connect(widget._environment_created)

    @xfail
    def test_ipython_option(self, env_tab, tmpfolder):
        widget, qtbot, tmpfile = env_tab
        pyver = 'python={0}'.format(self.package_version('python'))

        self.remove_env(widget)
        worker = widget.packages_widget.create_environment(name='navigatortest', packages=[pyver])
        worker.name = 'navigatortest'
        worker.communicate()  # run create
        widget._environment_created(worker, "", "")
        widget.menu_list.exec_ = lambda *args: True
        qtbot.mouseClick(widget.list_environments.currentItem().button_options, Qt.LeftButton)
        is_action_enabled = widget.menu_list.actions()[2].isEnabled()
        assert not is_action_enabled

        worker = widget.packages_widget.api.conda_install(name='navigatortest', pkgs=['jupyter-core'])
        worker.communicate()
        qtbot.mouseClick(widget.list_environments.currentItem().button_options, Qt.LeftButton)
        assert not widget.menu_list.actions()[2].isEnabled()

        worker = widget.packages_widget.api.conda_install(name='navigatortest', pkgs=['ipython'])
        worker.communicate()
        qtbot.mouseClick(widget.list_environments.currentItem().button_options, Qt.LeftButton)
        assert widget.menu_list.actions()[2].isEnabled()

        worker = widget.packages_widget.remove_environment(name='navigatortest')
        worker.communicate()  # run create

        self.remove_env(widget)



# Standard library imports
import json
import os
import webbrowser

# Local imports
from anaconda_navigator.api.external_apps.base import BaseInstallableApp
from anaconda_navigator.config import BITS_32, BITS_64, LINUX, MAC
from anaconda_navigator.static import images
from anaconda_navigator.utils.logs import logger


class BasePyCharmApp(BaseInstallableApp):
    def __init__(self, edition, **kwargs):
        self.edition = edition

        super(BasePyCharmApp,
              self).__init__(filename='pycharm', windows_folder_name='JetBrains', needs_license=False, **kwargs)

        self.is_available = BITS_64

    def get_version_from_product_info_file(self):
        # for LINUX and WIN 'product-info.json' is located in app root directory, for MAC - not
        path = os.path.join('Contents', 'Resources') if MAC else ''
        version_path = os.path.join(self.app_directory_path, path, "product-info.json")
        if os.path.exists(version_path):
            with open(version_path, "r") as file:
                return json.load(file)['version']
        else:
            logger.debug('Can\'t find PyCharm version file by path: {0}'.format(version_path))
            return ''

    @property
    def executable(self) -> str:
        """Return an executable command to launch the app."""

        if MAC:
            executable_file = os.path.join(self.app_directory_path, "Contents", "MacOS", self.filename)
            executable_command = f'"{executable_file}"'
        elif LINUX:
            executable_file = os.path.join(self.app_directory_path, "bin", self.filename + ".sh")
            executable_command = f'sh "{executable_file}"'
        else:
            filename = f'{self.filename}{"32" if BITS_32 else "64"}.exe'
            executable_file = os.path.join(self.app_directory_path, "bin", filename)
            executable_command = f'"{executable_file}"'

        if os.path.exists(executable_file):
            return executable_command

        return ''

    def _get_linux_installation_directory(self) -> str:
        root_dirs = ['/opt']

        if os.path.exists('/snap'):
            root_dirs.append('/snap')

        for root in root_dirs:
            for dir_name in os.listdir(root):
                if dir_name.startswith(f'{self.filename}-{self.edition}'):
                    # using Snap user can install several versions, 'current' is a link to newest
                    return os.path.join(root, dir_name, 'current' if root == '/snap' else '')
        return ''

    def _get_windows_installation_directory(self) -> str:
        from anaconda_navigator.external.knownfolders import FOLDERID, get_folder_path

        edition_dir_map = {'professional': 'PyCharm 20', 'community': 'PyCharm Community'}

        if BITS_32:
            program_file_dir = get_folder_path(FOLDERID.ProgramFilesX86)[0]
        else:
            program_file_dir = get_folder_path(FOLDERID.ProgramFilesX64)[0]

        jet_brains_dir = os.path.join(program_file_dir, self.windows_folder_name)

        if os.path.exists(jet_brains_dir):
            for dir_name in os.listdir(jet_brains_dir):
                if dir_name.startswith(edition_dir_map[self.edition]):
                    return os.path.join(jet_brains_dir, dir_name)

        return ''

    def _get_dummy_worker(self):
        return self._process_api.create_process_worker(['python', '--version'])

    @property
    def versions(self):
        """Return the currently installed version of the application."""
        # for the case when app is not installed, but is available to install
        if not self.is_installed:
            return ['']

        return [self.get_version_from_product_info_file()]

    def update_config(self, prefix):
        pass

    def install_extensions(self):
        return self._get_dummy_worker()


class PyCharmProApp(BasePyCharmApp):
    def __init__(self, **kwargs):
        super(PyCharmProApp, self).__init__(
            app_name='pycharm_pro',
            mac_name='PyCharm.app',
            display_name='PyCharm Professional',
            description='A full-fledged IDE by JetBrains for both Scientific '
            'and Web Python development. Supports HTML, JS, and SQL.',
            image_path=images.PYCHARM_ICON_1024_PATH,
            edition='professional',
            is_installation_enabled=True,
            **kwargs
        )

    def install(self):
        webbrowser.open_new_tab('https://www.anaconda.com/pycharm_navigator')


class PyCharmCEApp(BasePyCharmApp):
    def __init__(self, **kwargs):
        super(PyCharmCEApp, self).__init__(
            app_name='pycharm_ce',
            mac_name='PyCharm CE.app',
            display_name='PyCharm Community',
            description='An IDE by JetBrains for pure Python development. '
            'Supports code completion, listing, and debugging.',
            image_path=images.PYCHARM_CE_ICON_1024_PATH,
            edition='community',
            is_installation_enabled=False,
            **kwargs
        )
